@@ -1,44 +1,48 @@
 import os
 import sys
 
+import sphinx_rtd_theme
+
+from cloudinit import version
+
 # If extensions (or modules to document with autodoc) are in another directory,
 # add these directories to sys.path here. If the directory is relative to the
 # documentation root, use os.path.abspath to make it absolute, like shown here.
-sys.path.insert(0, os.path.abspath('../../'))
-sys.path.insert(0, os.path.abspath('../'))
-sys.path.insert(0, os.path.abspath('./'))
-sys.path.insert(0, os.path.abspath('.'))
+sys.path.insert(0, os.path.abspath("../../"))
+sys.path.insert(0, os.path.abspath("../"))
+sys.path.insert(0, os.path.abspath("./"))
+sys.path.insert(0, os.path.abspath("."))
 
-from cloudinit import version
-from cloudinit.config.schema import get_schema_doc
 
-# Supress warnings for docs that aren't used yet
+# Suppress warnings for docs that aren't used yet
 # unused_docs = [
 # ]
 
 # General information about the project.
-project = 'cloud-init'
-copyright = '2020, Canonical Ltd.'
+project = "cloud-init"
+copyright = "2022, Canonical Ltd."
 
 # -- General configuration ----------------------------------------------------
 
 # If your documentation needs a minimal Sphinx version, state it here.
-# needs_sphinx = '1.0'
+needs_sphinx = "4.0"
 
 # Add any Sphinx extension module names here, as strings. They can be
 # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
 extensions = [
-    'm2r',
-    'sphinx.ext.autodoc',
-    'sphinx.ext.autosectionlabel',
-    'sphinx.ext.viewcode',
+    "m2r2",
+    "sphinx_rtd_theme",
+    "sphinx_panels",
+    "sphinx.ext.autodoc",
+    "sphinx.ext.autosectionlabel",
+    "sphinx.ext.viewcode",
 ]
 
 # The suffix of source filenames.
-source_suffix = '.rst'
+source_suffix = ".rst"
 
 # The master toctree document.
-master_doc = 'index'
+master_doc = "index"
 
 # The version info for the project you're documenting, acts as replacement for
 # |version| and |release|, also used in various other places throughout the
@@ -47,7 +51,7 @@ version = version.version_string()
 release = version
 
 # Set the default Pygments syntax
-highlight_language = 'yaml'
+highlight_language = "yaml"
 
 # List of patterns, relative to source directory, that match files and
 # directories to ignore when looking for source files.
@@ -61,17 +65,12 @@ show_authors = False
 
 # The theme to use for HTML and HTML Help pages.  See the documentation for
 # a list of builtin themes.
-html_theme = 'sphinx_rtd_theme'
+html_theme = "sphinx_rtd_theme"
 
 # The name of an image file (relative to this directory) to place at the top
 # of the sidebar.
-html_logo = 'static/logo.png'
+html_logo = "static/logo.png"
 
-def generate_docstring_from_schema(app, what, name, obj, options, lines):
-    """Override module docs from schema when present."""
-    if what == 'module' and hasattr(obj, "schema"):
-        del lines[:]
-        lines.extend(get_schema_doc(obj.schema).split('\n'))
-
-def setup(app):
-    app.connect('autodoc-process-docstring', generate_docstring_from_schema)
+# Make sure the target is unique
+autosectionlabel_prefix_document = True
+autosectionlabel_maxdepth = 2
